	.model	small,c
; THIS PROGRAM OFFERED BY DOUG FOR DEVELOPERS; ALLOWS THEM TO DETECT STKR VOLUME
; The following code assumes small C model (DS=SS).
;
;
; Intended usage:
;
;	if ((detect_stacker() != 0) &&
;	    (detect_stacker_volume(driveNumber) == 0)
;	  	printf("Drive %c is a Stacker volume",'A'+driveNumber);
;	else
;	  	printf("Drive %c is a not a Stacker volume",'A'+driveNumber);
;
;
; Changes:
;  05/01/91 -	detect_stacker() returns version number
;  		detect_stacker_volume2() returns BL=Stacker unit #
;		added stacker_dd_pointer()
;
	.DATA
ST_PTR	DD	0			;pointer to Stacker (0-->not there)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
	.CODE
	
	mov	ax,5
	push	ax
	call	stacker_dd_pointer
	pop	cx
	mov	ah,4ch
	int	21h
;
; C declaration:  int detect_stacker(void); 
;
; Input: None
; Output:AX=0  --> Stacker not installed
;	 AX<>0 --> Stacker version*100  (e.g.  1.00 --> 0064H)
;	
;	Stacker is detected by making an INT 25H call with invalid
;	parameters.
;
detect_stacker proc
	push	bp
	push	si
	push	di

	sub	sp,1024			;use the stack as a temp buffer

	mov	ax,0cdcdh		;invalid drive #
	mov	bx,sp			;DS:BX --> buffer for INT 25H
	mov	cx,1
	xor	dx,dx			;read boot sector of invalid drive
	mov	word ptr [bx],dx	;clear the "return" value
	push	ds			;(set ES:BP to fix DOS 3 INT 25H bug)
	pop	es			
	mov	bp,bx
	int	25H
	pop	cx			;get rid of flags

	xor	ax,ax			;default is No stacker
	mov	bx,sp			;point at result
	cmp	word ptr [bx],0CDCDH	;Stacker INT 25 fills in some fields.
	jnz	gotres			;Make sure they all match
	cmp	word ptr 2[bx],1
	jnz	gotres
	les	di,4[bx]		;pointer into Stacker
	cmp	word ptr es:[di],0A55AH	;must find signature
	jnz	gotres
	mov	word ptr st_ptr  ,di	;save pointer to show it's found
	mov	word ptr st_ptr+2,es
	mov	ax,es:[di+2]		;stacker version * 100
gotres:
	add	sp,1024
	pop	di			;restore regs
	pop	si
	pop	bp
	ret
detect_stacker	endp


;
; C declaration:  int detect_stacker_volume(int driveno);
;
; Input: driveno: 0=A, 1=B, 2=C, etc.
;
; Output:AX=0 --> Is a Stacker volume
;	 AX=1 --> Is not a Stacker volume
;	
detect_stacker_volume proc driveno:word
	cmp	word ptr st_ptr+2,0	;already found Stacker?
	jnz	stacker_fnd
	call	detect_stacker		;is not, try again
	xor	ax,ax			;return 0 if not found
	cmp	ax,word ptr st_ptr+2	;is it ther
	jnz	stacker_fnd
notstk:	ret
	; Here if Stacker IS installed.
stacker_fnd:
	mov	ax,4404H		;do an ioctl call
	mov	bx,driveno
	inc	bx			;adjust for default
	mov	cx,4			;read 4 bytes = Stacker pointer
	push	cx			;make room on the stack
	push	cx
	mov	dx,sp			;ds:dx-->return area
	int	21h			;if Stacker drive, will return
	pop	cx			;pointer to Stacker
	pop	bx			;cx:bx=pointer returned
	mov	ax,0
	jc	notstk			;no ioctl support --> not Stacker
	cmp	bx,word ptr st_ptr+2	;does the pointer match?
	jnz	notstk
	cmp	cx,word ptr st_ptr
	jnz	notstk
	inc	ax			;if so, we have a Stacker volume
	ret
detect_stacker_volume endp


;
; C declaration:  int detect_stacker_volume2(int driveno);
;
; Input: driveno: 0=A, 1=B, 2=C, etc.
;
; Output:AX=0 --> Is a Stacker volume
;	 AX=1 --> Is not a Stacker volume
;	 BL = Stacker unit number if AX=1
;	
; Notes: If a multitasking environment is present (such as Windows), you 
;	 must make this a critical section of code!
;
;	 This method uses the removeable media ioctl call to detect 
;	 Stacker volumes.  It does NOT work under DR DOS 5.0, since that
;	 version of DOS does NOT pass these calls through to the Stacker
;	 device driver.
;

UNIT_OFFS equ	3EH			;offset with Stacker of 

detect_stacker_volume2 proc driveno:word
	uses	di,si
	cmp	word ptr st_ptr+2,0	;already found Stacker?
	jnz	stacker_fnd2
	call	detect_stacker		;is not, try again
	xor	ax,ax			;return 0 if not found
	cmp	ax,word ptr st_ptr+2	;is it ther
	jnz	stacker_fnd2
notstk2:ret
	; Here if Stacker IS installed.
stacker_fnd2:
	les	di,st_ptr
	mov	byte ptr es:UNIT_OFFS[di],0FFH	;set unit #

	mov	ax,4408H		;do an ioctl call (removeable?)
	mov	bx,driveno
	inc	bx			;adjust for default
	int	21h			;if Stacker drive, will return
	mov	ax,0
	jc	notstk2			;no ioctl support --> not Stacker

	les	di,st_ptr		;see if unit # changed
	mov	bl,byte ptr es:UNIT_OFFS[di]
	cmp	bl,0FFH
	jz	notstk2

	inc	ax			;if so, we have a Stacker volume
	ret
detect_stacker_volume2 endp



STACKER_DD struc
dv_strategy	dd	1 dup(?)	;physical device driver strategy addr
dv_interrupt	dd	1 dup(?)	;physical device driver interrupt addr
dv_att		dw	1 dup(?)	;device driver header attributes
dv_cluster0	dw	1 dup(?)	;first file cluster of
dv_log2		db	1 dup(?)	;LOG base 2 of physical bytes/sector
dv_unit		db	1 dup(?)	;physical device driver unit number
STACKER_DD ends

STACKER_UNIT_DD_OFFSET	equ	015CH

;
; C declaration:  int stacker_dd_pointer(int driveno);
;
; Input: driveno: 0=A, 1=B, 2=C, etc.
;
; Output:DX:AX = far pointer to the STACKER_DD struc (0:0 if error)
;	
; Notes: If a multitasking environment is present (such as Windows), you 
;	 must make this a critical section of code!

stacker_dd_pointer proc driveno:word
	uses	di,si
	push	driveno
	call	detect_stacker_volume2
	pop	cx			;get rid of driveNo on stack
	cwd
	or	ax,ax
	jnz	isStacker
	ret
isStacker:	
	les	di,st_ptr
	mov	di,es:4[di]		;get unit table pointer
	xor	bh,bh
	add	bx,bx			;multiply unit # by two
	mov	ax,es:[di+bx]		;dx:ax = per unit pointer
	add	ax,STACKER_UNIT_DD_OFFSET
	mov	dx,es			;return pointer
	ret
stacker_dd_pointer endp

	end
